import tweepy
import pandas as pd

class GetTweet:
    def __init__(self, CONSUMER_KEY, CONSUMER_SECRET, ACCESS_TOKEN, ACCESS_TOKEN_SECRET, proxy=None):
        auth = tweepy.OAuthHandler(CONSUMER_KEY, CONSUMER_SECRET)
        auth.set_access_token(ACCESS_TOKEN, ACCESS_TOKEN_SECRET)

        if proxy:
            self.api = tweepy.API(auth, wait_on_rate_limit = True, proxy=proxy)
        else:
            self.api = tweepy.API(auth, wait_on_rate_limit = True, retry_count=5)

    # get Target's tweets
    def get_tweets_target(self, target):
        tweets = {"tweet_id": [], "created_at": [], "text": [], "favorite_count": [], "retweet_count": []}
        for tweet in tweepy.Cursor(self.api.user_timeline, screen_name = target, exclude_replies = True).items():
            tweets["tweet_id"].append(tweet.id)
            tweets["created_at"].append(tweet.created_at)
            tweets["text"].append(tweet.text)
            tweets["favorite_count"].append(tweet.favorite_count)
            tweets["retweet_count"].append(tweet.retweet_count)

        return pd.DataFrame(tweets)

    # search for keywords
    def get_tweets_keyword(self, keyword):
        tweets = {
            "tweet_id": [],
            "created_at": [],
            "screen_name": [],
            "user_name": [],
            "user_description": [],
            "followers_count": [],
            "following_count": [],
            "text": [],
            "favorite_count": [],
            "retweet_count": []
        }
        for tweet in tweepy.Cursor(self.api.search, q=keyword, include_entities=True, tweet_mode='extended', lang='ja').items():
            tweets["tweet_id"].append(tweet.id)
            tweets["created_at"].append(tweet.created_at)
            tweets["screen_name"].append(tweet.user.screen_name)
            tweets["user_name"].append(tweet.user.name)
            tweets["user_description"].append(tweet.user.description)
            tweets["followers_count"].append(tweet.user.followers_count)
            tweets["following_count"].append(tweet.user.friends_count)
            tweets["text"].append(tweet.full_text)
            tweets["favorite_count"].append(tweet.favorite_count)
            tweets["retweet_count"].append(tweet.retweet_count)

        return pd.DataFrame(tweets)



import pandas as pd
import json
import re
import emoji
import mojimoji
import neologdn

class CleansingTweets:

    # replace and\s to space
    def cleansing_space(self, text):
        return re.sub("\u3000|\s", " ", text)

    # remove hashtags
    def cleansing_hash(self, text):
        return re.sub("#[^\s]+", "", text)

    # remove URLs
    def cleansing_url(self, text):
        return re.sub(r"(https?|ftp)(:\/\/[-_\.!~*\'()a-zA-Z0-9;\/?:\@&=\+$,%#]+)", "" , text)

    # remove pictograph
    def cleansing_emoji(self, text):
        return ''.join(c for c in text if c not in emoji.UNICODE_EMOJI)

    # remove mention
    def cleansing_username(self, text):
        return re.sub(r"@([A-Za-z0-9_]+) ", "", text)

    #  remove image string
    def cleansing_picture(self, text):
        return re.sub(r"pic.twitter.com/[-_\.!~*\'()a-zA-Z0-9;\/?:\@&=\+$,%#]*", "" , text)

    # unify characters.
    def cleansing_unity(self, text):
        text = text.lower()
        text = mojimoji.zen_to_han(text, kana=True)
        text = mojimoji.han_to_zen(text, digit=False, ascii=False)
        return text

    # replace number to zero
    def cleansing_num(self, text):
        text = re.sub(r'\d+', "0", text)
        return text

    # remove rt
    def cleansing_rt(self, text):
        return re.sub(r"RT @[-_\.!~*\'()a-zA-Z0-9;\/?:\@&=\+$,%#]*?: ", "" , text)

    def cleansing_text(self, text):
        text = self.cleansing_rt(text)
        text = self.cleansing_hash(text)
        text = self.cleansing_space(text)
        text = self.cleansing_url(text)
        text = self.cleansing_emoji(text)
        text = self.cleansing_username(text)
        text = self.cleansing_picture(text)
        text = self.cleansing_unity(text)
        text = self.cleansing_num(text)
        text = neologdn.normalize(text)
        return text

    def cleansing_df(self, df, subset_cols=["text"]):
        if "text" in subset_cols:
            # remove duplicates (because they might be RT.)
            df = df.drop_duplicates(subset="text", keep=False)

        df_copy = df.copy()

        for col in subset_cols:
            # cleansing
            df_copy[col] = df[col].apply(lambda x: self.cleansing_text(x))
            
        if "text" in subset_cols:
            # remove duplicates
            df_copy = df_copy.drop_duplicates(subset="text", keep=False).reset_index(drop=True)

        return df_copy
